@@ -1,3 +1,14 @@
+# -*- coding: utf-8 -*-
+"""
+Module collecting classes and functions for dealing with (multiphase) chemical
+equilibria.
+
+.. Note::
+
+  This module is provisional at the moment, i.e. the API is not stable and may
+  break without a deprecation cycle.
+
+"""
 from __future__ import division, absolute_import
 
 import math
@@ -10,6 +21,7 @@ from operator import mul, add
 import numpy as np
 
 from .chemistry import ReactionSystem, equilibrium_quotient
+from .util.pyutil import deprecated
 
 
 def reducemap(args, reduce_op, map_op):
@@ -153,6 +165,16 @@ class _NumSys(object):
         self.exp = exp
         self.precipitates = precipitates
 
+    def _get_A_ks(self, eq_params):
+        non_precip_rids = self.eqsys.non_precip_rids(self.precipitates)
+        return self.eqsys.stoichs_constants(
+            self.eqsys.eq_constants(non_precip_rids, eq_params, self.small),
+            self.rref_equil, ln=self.ln, exp=self.exp,
+            non_precip_rids=non_precip_rids)
+
+    def _inits_and_eq_params(self, params):
+        return params[:self.eqsys.ns], params[self.eqsys.ns:]
+
 
 class NumSysLin(_NumSys):
 
@@ -162,22 +184,23 @@ class NumSysLin(_NumSys):
 
     def f(self, yvec, params):
         from pyneqsys.symbolic import linear_exprs
-        init_concs, eq_params = params[:self.eqsys.ns], params[self.eqsys.ns:]
-        non_precip_rids = self.eqsys.non_precip_rids(self.precipitates)
-        A, ks = self.eqsys.stoichs_constants(
-            self.eqsys.eq_constants(non_precip_rids, eq_params, self.small),
-            self.rref_equil, ln=self.ln, exp=self.exp,
-            non_precip_rids=non_precip_rids)
+        init_concs, eq_params = self._inits_and_eq_params(params)
+        A, ks = self._get_A_ks(eq_params)
         # yvec == C
         f_equil = [q/k - 1 if k != 0 else q for q, k
                    in zip(prodpow(yvec, A), ks)]
         B, comp_nrs = self.eqsys.composition_balance_vectors()
         f_preserv = linear_exprs(B, yvec, mat_dot_vec(B, init_concs),
                                  rref=self.rref_preserv)
-        # import sympy as sp
-        # f3 = [sp.Piecewise((yi**2, yi < 0), (0, True)) for yi in yvec]
-        # f3 = [sp.ITE(yi < 0, yi**2, 0) for yi in yvec]
-        return f_equil + f_preserv  # + f3
+        return f_equil + f_preserv
+
+
+class _NumSysLinNegPenalty(NumSysLin):
+
+    def f(self, yvec, params):
+        import sympy as sp
+        f_penalty = [sp.Piecewise((yi**2, yi < 0), (0, True)) for yi in yvec]
+        return super(_NumSysLinNegPenalty, self).f(yvec, params) + f_penalty
 
 
 class NumSysLinRel(NumSysLin):
@@ -201,7 +224,7 @@ class NumSysLinRel(NumSysLin):
 
 class NumSysSquare(NumSysLin):
 
-    small = 1e-40
+    small = 1e-35
 
     def pre_processor(self, x, params):
         return (np.sqrt(np.abs(x)), params)
@@ -254,17 +277,13 @@ class NumSysLog(_NumSys):
     def internal_x0_cb(self, init_concs, params):
         # return [1]*len(init_concs)
         return [0.1]*len(init_concs)
-        # np.log(np.abs(init_concs))/10 # [0]*len(init_concs)
 
     def f(self, yvec, params):
         from pyneqsys.symbolic import linear_exprs
-        init_concs, eq_params = params[:self.eqsys.ns], params[self.eqsys.ns:]
-        non_precip_rids = self.eqsys.non_precip_rids(self.precipitates)
-        A, ks = self.eqsys.stoichs_constants(
-            self.eqsys.eq_constants(non_precip_rids, eq_params, self.small),
-            self.rref_equil, ln=self.ln, exp=self.exp,
-            non_precip_rids=non_precip_rids)
-        f_equil = mat_dot_vec(A, yvec, [-self.ln(k) for k in ks])  # y == ln(C)
+        init_concs, eq_params = self._inits_and_eq_params(params)
+        A, ks = self._get_A_ks(eq_params)
+        # yvec == ln(C)
+        f_equil = mat_dot_vec(A, yvec, [-self.ln(k) for k in ks])
         B, comp_nrs = self.eqsys.composition_balance_vectors()
         f_preserv = linear_exprs(B, list(map(self.exp, yvec)),
                                  mat_dot_vec(B, init_concs),
@@ -336,12 +355,24 @@ class EqSystem(ReactionSystem):
                 np.dot(A, self.as_per_substance_array(concs).T),
                 np.dot(A, self.as_per_substance_array(init_concs).T))
 
+    def other_phase_species_idxs(self, phase_idx=0):
+        return [idx for idx, s in enumerate(
+            self.substances.values()) if s.phase_idx != phase_idx]
+
     @property
+    @deprecated(last_supported_version='0.3.1', will_be_missing_in='0.4.0',
+                use_instead=other_phase_species_idxs)
     def precipitate_substance_idxs(self):
         return [idx for idx, s in enumerate(
             self.substances.values()) if s.precipitate]
 
+    def phase_transfer_reaction_idxs(self, phase_idx=0):
+        return [idx for idx, rxn in enumerate(self.rxns)
+                if rxn.has_precipitates(self.substances)]
+
     @property
+    @deprecated(last_supported_version='0.3.1', will_be_missing_in='0.4.0',
+                use_instead=phase_transfer_reaction_idxs)
     def precipitate_rxn_idxs(self):
         return [idx for idx, rxn in enumerate(self.rxns)
                 if rxn.has_precipitates(self.substances)]
@@ -412,30 +443,20 @@ class EqSystem(ReactionSystem):
                                        rref_preserv=False, NumSys=NumSysLin):
         from pyneqsys import ConditionalNeqSys, ChainedNeqSys
 
-        try:
-            NumSys[0]
-        except TypeError:
-            NumSys = (NumSys,)
-
         def factory(conds):
             return ChainedNeqSys([self._SymbolicSys_from_NumSys(
                 NS, conds, rref_equil, rref_preserv) for NS in NumSys])
 
         cond_cbs = [(self._fw_cond_factory(ri),
                      self._bw_cond_factory(ri, NumSys[0].small)) for
-                    ri in self.precipitate_rxn_idxs]
+                    ri in self.phase_transfer_reaction_idxs()]
         return ConditionalNeqSys(cond_cbs, factory)
 
     def get_neqsys_chained_conditional(self, init_concs, rref_equil=False,
                                        rref_preserv=False,
-                                       NumSys=NumSysLin, **kwargs):
+                                       NumSys=NumSysLin):
         from pyneqsys import ConditionalNeqSys, ChainedNeqSys
 
-        try:
-            NumSys[0]
-        except TypeError:
-            NumSys = (NumSys,)
-
         def mk_factory(NS):
             def factory(conds):
                 return self._SymbolicSys_from_NumSys(NS, conds, rref_equil,
@@ -446,16 +467,39 @@ class EqSystem(ReactionSystem):
             [ConditionalNeqSys(
                 [(self._fw_cond_factory(ri),
                   self._bw_cond_factory(ri, NS.small)) for
-                 ri in self.precipitate_rxn_idxs],
+                 ri in self.phase_transfer_reaction_idxs()],
                 mk_factory(NS)
             ) for NS in NumSys])
 
-    # get_neqsys = get_neqsys_conditional_chained  #
-    get_neqsys = get_neqsys_chained_conditional  # fails more often
+    def get_neqsys_static_conditions(self, init_concs, rref_equil=False,
+                                     rref_preserv=False,
+                                     NumSys=NumSysLin, precipitates=None):
+        if precipitates is None:
+            precipitates = (False,)*len(self.phase_transfer_reaction_idxs())
+        from pyneqsys import ChainedNeqSys
+        return ChainedNeqSys([self._SymbolicSys_from_NumSys(
+            NS, precipitates, rref_equil, rref_preserv) for NS in NumSys])
+
+    def get_neqsys(self, neqsys_type, init_concs, NumSys=NumSysLin, **kwargs):
+        new_kw = {'rref_equil': False, 'rref_preserv': False}
+        if neqsys_type == 'static_conditions':
+            new_kw['precipitates'] = None
+        for k in new_kw:
+            if k in kwargs:
+                new_kw[k] = kwargs.pop(k)
+
+        try:
+            NumSys[0]
+        except TypeError:
+            new_kw['NumSys'] = (NumSys,)
+        else:
+            new_kw['NumSys'] = NumSys
+
+        return getattr(self, 'get_neqsys_' + neqsys_type)(init_concs, **new_kw)
 
     def non_precip_rids(self, precipitates):
         return [idx for idx, precip in zip(
-            self.precipitate_rxn_idxs, precipitates) if not precip]
+            self.phase_transfer_reaction_idxs(), precipitates) if not precip]
 
     def _result_is_sane(self, init_concs, x):
         sc_upper_bounds = np.array(self.upper_conc_bounds(init_concs))
@@ -469,18 +513,17 @@ class EqSystem(ReactionSystem):
             return False
         return True
 
-    def root(self, init_concs, x0=None, neqsys=None, NumSys=NumSysLin,
-             **kwargs):
+    def root(self, init_concs, x0=None, neqsys=None, NumSys=NumSysLog,
+             neqsys_type='chained_conditional', **kwargs):
         init_concs = self.as_per_substance_array(init_concs)
         params = np.concatenate((init_concs, [float(elem) for elem
                                               in self.eq_constants()]))
         if neqsys is None:
             neqsys = self.get_neqsys(
-                init_concs,
+                neqsys_type, init_concs, NumSys=NumSys,
                 rref_equil=kwargs.pop('rref_equil', False),
                 rref_preserv=kwargs.pop('rref_preserv', False),
-                NumSys=NumSys
-            )
+                precipitates=kwargs.pop('precipitates', None))
         if x0 is None:
             x0 = init_concs
         x, sol = neqsys.solve(x0, params, **kwargs)
@@ -505,8 +548,11 @@ class EqSystem(ReactionSystem):
             return [s.name for s in self.substances.values()]
 
     def roots(self, init_concs, varied_data, varied, x0=None,
-              NumSys=NumSysLin, plot_kwargs=None, **kwargs):
+              NumSys=NumSysLog, plot_kwargs=None,
+              neqsys_type='chained_conditional', **kwargs):
         """
+        Parameters
+        ----------
         init_concs: array or dict
         varied_data: array
         varied_idx: int or str
@@ -518,6 +564,8 @@ class EqSystem(ReactionSystem):
             are intercepted here:
                 latex_names: bool (default: False)
                 conc_unit_str: str (default: 'M')
+        neqsys_type: str
+            what method to use for NeqSys construction (get_neqsys_*)
         \*\*kwargs:
             kwargs passed on to py:meth:`pyneqsys.NeqSys.solve_series`
         """
@@ -528,23 +576,21 @@ class EqSystem(ReactionSystem):
             if 'ax' not in plot_kwargs:
                 plot_kwargs['ax'] = self._get_default_plot_ax()
 
-        new_kwargs = kwargs.copy()
         init_concs = self.as_per_substance_array(init_concs)
         neqsys = self.get_neqsys(
-            init_concs,
-            rref_equil=new_kwargs.pop('rref_equil', False),
-            rref_preserv=new_kwargs.pop('rref_preserv', False),
-            NumSys=NumSys
-        )
+            neqsys_type, init_concs, NumSys=NumSys,
+            rref_equil=kwargs.pop('rref_equil', False),
+            rref_preserv=kwargs.pop('rref_preserv', False),
+            precipitates=kwargs.pop('precipitates', None))
         if x0 is None:
             x0 = init_concs
 
         if _plot:
             cb = neqsys.solve_and_plot_series
-            if 'plot_kwargs' not in new_kwargs:
-                new_kwargs['plot_kwargs'] = {}
-            if 'labels' not in new_kwargs['plot_kwargs']:
-                new_kwargs['plot_kwargs']['labels'] = (
+            if 'plot_kwargs' not in kwargs:
+                kwargs['plot_kwargs'] = {}
+            if 'labels' not in kwargs['plot_kwargs']:
+                kwargs['plot_kwargs']['labels'] = (
                     self.substance_labels(latex_names))
         else:
             cb = neqsys.solve_series
@@ -552,7 +598,7 @@ class EqSystem(ReactionSystem):
         params = np.concatenate((init_concs, self.eq_constants()))
         xvecs, info_dicts = cb(
             x0, params, varied_data, self.as_substance_index(varied),
-            propagate=False, **new_kwargs)
+            propagate=False, **kwargs)
         sanity = [self._result_is_sane(init_concs, x) for x in xvecs]
 
         if _plot:
