@@ -1,7 +1,15 @@
 # -*- coding: utf-8 -*-
+"""
+General utilities and exceptions.
+"""
 from __future__ import (absolute_import, division, print_function)
 
 from collections import namedtuple, Mapping
+import os
+import warnings
+
+from .. import __url__
+from .deprecation import Deprecation
 
 
 def defaultnamedtuple(typename, field_names, defaults=()):
@@ -9,12 +17,12 @@ def defaultnamedtuple(typename, field_names, defaults=()):
 
     Parameters
     ----------
-    typename: string
-         the name of the class
-    field_names: str or iterable
-        an iterable of splitable string
-    defaults: iterable
-        default values for field_names, counting [-len(defaults):]
+    typename : string
+        The name of the class.
+    field_names : str or iterable
+        An iterable of splitable string.
+    defaults : iterable
+        Default values for ``field_names``, counting ``[-len(defaults):]``.
 
     Examples
     --------
@@ -28,6 +36,7 @@ def defaultnamedtuple(typename, field_names, defaults=()):
     Returns
     -------
     A new tuple subclass named ``typename``
+
     """
     Tuple = namedtuple(typename, field_names)
     Tuple.__new__.__defaults__ = (None,) * len(Tuple._fields)
@@ -38,3 +47,20 @@ def defaultnamedtuple(typename, field_names, defaults=()):
         defaults = (None,)*nmissing + tuple(defaults)
         Tuple.__new__.__defaults__ = tuple(Tuple(*defaults))
     return Tuple
+
+
+class NoConvergence(Exception):
+    pass  # why is this not in the Python standard library!?
+
+
+class ChemPyDeprecationWarning(DeprecationWarning):
+    pass
+
+
+def deprecated(*args, **kwargs):
+    """ Helper to :class:`Deprecation` for using ChemPyDeprecationWarning. """
+    return Deprecation(*args, issues_url=__url__ + '/issues/%s/',
+                       warning=ChemPyDeprecationWarning, **kwargs)
+
+warnings.simplefilter(os.environ.get('CHEMPY_DEPRECATION_FILTER', 'once'),
+                      ChemPyDeprecationWarning)
